<?php
namespace metagen_cs_ecs;

function supported_tokens()
{
  return [
    'ecs_serializable',
    'ecs_gen_component',
    'ecs_gen_authoring',
    'ecs_tag',
  ];
}

function get_twig_inc()
{
  return __DIR__ ."/../tpl";
}

function generate_ecs_components(\mtgMetaInfo $meta)
{
  global $GAME_ROOT;

  $twig = \metagen_cs\get_twig(array(get_twig_inc()));
  $twig->setCache($GAME_ROOT . "/build/twig/metagen_cs_ecs");
  $gen_components = new \mtgMetaInfo();
  $lock_file = "$GAME_ROOT/unity/Packages/metagen_autogen/Runtime/code/ecs/ecs.lock";

  foreach ($meta->getUnits() as $unit)
  {
    if(!$unit->object->hasToken('ecs_gen_component') || !($unit->object instanceof \mtgUserType))
      continue;
    $gen_components->addUnit($unit);
  }

  if(!\taskman\need_to_regen($lock_file, get_meta_units_files($gen_components)))
  {
    return;
  }
  \taskman\rrmdir("$GAME_ROOT/unity/Packages/metagen_autogen/Runtime/code/ecs/");
  \taskman\ensure_write($lock_file, "");
  touch($lock_file);

  foreach ($gen_components->getUnits() as $unit)
  {
    $unit_name = $unit->object->getName();
    $filePath = str_replace(".", "/", $unit_name);
    $file = "$GAME_ROOT/unity/Packages/metagen_autogen/Runtime/code/ecs/$filePath.cs";
    $name_components = explode('.', $unit_name);
    $class_name = end($name_components);
    if($class_name != $unit_name)
      $namespace = str_replace(".$class_name", "", $unit_name);
    else
      $namespace = "";

    // Костыль. Убрать, когда будет поддержка namespace в bhl_bind
    if(empty($namespace))
      $namespace = "ecs";

    \taskman\ensure_write_if_differs(
      $file,
      $twig->render('codegen_ecs_component.twig', [
        'namespace' => $namespace,
        'class_name' => $class_name,
        'obj' => $unit->object,
        'imports' => [
          'System.Collections',
          'System.Collections.Generic',
          'System',
          'BitGames',
          'BitGames.Autogen',
          'UnityEngine',
          'bhl',
          'Leopotam.Ecs'
        ],
      ])
    );

    $file_authoring = "$GAME_ROOT/unity/Packages/metagen_autogen/Runtime/code/ecs/{$filePath}AuthoringComponent.cs";
    if($unit->object->hasToken('ecs_gen_authoring'))
    {
      \taskman\ensure_write_if_differs(
        $file_authoring,
        $twig->render('codegen_ecs_authoring.twig', [
          'namespace' => $namespace,
          'class_name' => $class_name,
          'obj' => $unit->object
        ])
      );
    }
  }
}

function get_meta_units_files(\mtgMetaInfo $meta)
{
  $files = array();
  foreach($meta->getUnits() as $u)
    $files[$u->file] = true;
  return array_keys($files);
}